@@ -73,6 +73,13 @@ pub struct NtContext {
     LdrpModuleBaseAddressIndex: RtlMutex<RTL_RB_TREE>,
 }
 
+unsafe impl Send for NtContext {}
+
+/// This allows us to hold a pointer across a wait point
+struct Module(*const u8);
+
+unsafe impl Send for Module {}
+
 impl NtContext {
     /// Gets the loaded NTDLL hash
     ///
@@ -173,12 +180,13 @@ impl NtContext {
             server_port
         ))?;
         let mut client = OffsetClient::new(channel.connect().await?);
-        let ntdll = unsafe { GetModuleHandleW(to_wide("ntdll").as_ptr()) as *const u8 };
+        // wrap this in a `Module` so we can send it across await points
+        let ntdll = Module(unsafe { GetModuleHandleW(to_wide("ntdll").as_ptr()) as *const u8 });
         let request = tonic::Request::new(OffsetsRequest {
-            ntdll_hash: NtContext::get_ntdll_hash(ntdll)?,
+            ntdll_hash: NtContext::get_ntdll_hash(ntdll.0)?,
         });
         let response = client.get_offsets(request).await?.into_inner();
-        Ok(NtContext::process_response(response, ntdll))
+        Ok(NtContext::process_response(response, ntdll.0))
     }
 
     /// Resolves the context used by the mapper, over a secure TLS connection
@@ -217,12 +225,13 @@ impl NtContext {
         ))?
         .tls_config(tls_config)?;
         let mut client = OffsetClient::new(channel.connect().await?);
-        let ntdll = unsafe { GetModuleHandleW(to_wide("ntdll").as_ptr()) as *const u8 };
+        // wrap this in a `Module` so we can send it across await points
+        let ntdll = Module(unsafe { GetModuleHandleW(to_wide("ntdll").as_ptr()) as *const u8 });
         let request = tonic::Request::new(OffsetsRequest {
-            ntdll_hash: NtContext::get_ntdll_hash(ntdll)?,
+            ntdll_hash: NtContext::get_ntdll_hash(ntdll.0)?,
         });
         let response = client.get_offsets(request).await?.into_inner();
-        Ok(NtContext::process_response(response, ntdll))
+        Ok(NtContext::process_response(response, ntdll.0))
     }
 
     /// Resolves the context used by the mapper
@@ -231,12 +240,13 @@ impl NtContext {
     ///
     /// `handler`: The local handler
     pub async fn resolve_local(handler: &OffsetHandler) -> anyhow::Result<NtContext> {
-        let ntdll = unsafe { GetModuleHandleW(to_wide("ntdll").as_ptr()) as *const u8 };
+        // wrap this in a `Module` so we can send it across await points
+        let ntdll = Module(unsafe { GetModuleHandleW(to_wide("ntdll").as_ptr()) as *const u8 });
         let request = tonic::Request::new(OffsetsRequest {
-            ntdll_hash: NtContext::get_ntdll_hash(ntdll)?,
+            ntdll_hash: NtContext::get_ntdll_hash(ntdll.0)?,
         });
         let response = handler.get_offsets(request).await?.into_inner();
-        Ok(NtContext::process_response(response, ntdll))
+        Ok(NtContext::process_response(response, ntdll.0))
     }
 }
 
@@ -355,6 +365,22 @@ impl<'a> PortableExecutable<'a> {
         self.added_to_index = true;
     }
 
+    /// Adds the module to the PEB structures
+    fn add_to_peb(&mut self) {
+        unsafe {
+            let peb = NtCurrentPeb();
+            let ldr = (*peb).Ldr;
+            InsertTailList(
+                &mut (*ldr).InLoadOrderModuleList,
+                &mut self.loader_entry.InLoadOrderLinks
+            );
+            InsertTailList(
+                &mut (*ldr).InMemoryOrderModuleList,
+                &mut self.loader_entry.InMemoryOrderLinks
+            );
+        }
+    }
+    
     /// Removes the module from the index red-black trees
     fn remove_from_index(&mut self) {
         unsafe {
@@ -432,13 +458,17 @@ impl<'a> PortableExecutable<'a> {
             .file
             .get_rva::<IMAGE_TLS_DIRECTORY>(tls_directory.VirtualAddress as isize)
             .ok_or(Error::TLSOutOfBounds)?;
-        let mut tls_callback = unsafe {
-            self.file
-                .get_rva::<Option<unsafe extern "stdcall" fn(PVOID, u32, PVOID)>>(
-                    ((*tls_directory).AddressOfCallBacks - self.file.contents() as u64) as isize,
-                )
-                .ok_or(Error::CallbackOutOfBounds)?
-        };
+        let tls_callbacks = unsafe { (*tls_directory).AddressOfCallBacks };
+        // it is possible that callbacks may not exist
+        if tls_callbacks == 0 {
+            return Ok(());
+        }
+        let mut tls_callback = self
+            .file
+            .get_rva::<Option<unsafe extern "stdcall" fn(PVOID, u32, PVOID)>>(
+                (tls_callbacks - self.file.contents() as u64) as isize,
+            )
+            .ok_or(Error::CallbackOutOfBounds)?;
         // execute all of the TLS callbacks
         unsafe {
             while (*tls_callback).is_some() {
@@ -528,6 +558,8 @@ impl<'a> PortableExecutable<'a> {
         self.add_to_hash_table();
         // add to the red-black trees for traversal
         self.add_to_index();
+        // add to the PEB linked lists
+        self.add_to_peb();
         Ok(())
     }
 
@@ -538,10 +570,7 @@ impl<'a> PortableExecutable<'a> {
     /// `path`: The path to the executable file
     ///
     /// `context`: The resolved Nt Context
-    pub fn load(
-        path: &str,
-        context: NtContext,
-    ) -> Result<PortableExecutable<'a>, anyhow::Error> {
+    pub fn load(path: &str, context: NtContext) -> Result<PortableExecutable<'a>, anyhow::Error> {
         PortableExecutable::load_as_primary(path, context, false)
     }
 
@@ -608,7 +637,7 @@ impl<'a> PortableExecutable<'a> {
                 FullDllName: UNICODE_STRING::default(),
                 BaseDllName: UNICODE_STRING::default(),
                 u2: LDR_DATA_TABLE_ENTRY_u2 { Flags: 0 },
-                ObsoleteLoadCount: 0,
+                ObsoleteLoadCount: u16::MAX,
                 TlsIndex: 0,
                 HashLinks: LIST_ENTRY::default(),
                 TimeDateStamp: 0,
@@ -952,11 +981,12 @@ mod test {
     #[test]
     #[serial]
     fn bad_section() {
-        let err: std::io::Error = PortableExecutable::load("test/badsection.exe", NT_CONTEXT.clone())
-            .err()
-            .unwrap()
-            .downcast()
-            .unwrap();
+        let err: std::io::Error =
+            PortableExecutable::load("test/badsection.exe", NT_CONTEXT.clone())
+                .err()
+                .unwrap()
+                .downcast()
+                .unwrap();
         assert_eq!(err.raw_os_error().unwrap(), ERROR_BAD_EXE_FORMAT as i32);
     }
 
